@@ -367,7 +367,8 @@ class HTTPResponse(HTTPMessage):
 		super().__init__()
 		self.status_code = status_code
 		self.status_name = STATUS_NAMES.get(status_code, "Unknown")
-		
+		self.upgrade = None
+	
 	def success(self):
 		return self.status_code // 100 == 2
 		
@@ -383,7 +384,7 @@ class HTTPResponse(HTTPMessage):
 		
 	def parse_start_line(self, line):
 		fields = line.split(maxsplit=2)
-		if len(fields) != 3:
+		if len(fields) not in [2, 3]:
 			raise HTTPError("Failed to parse HTTP response start line")
 		
 		self.version = fields[0]
@@ -393,7 +394,10 @@ class HTTPResponse(HTTPMessage):
 			raise HTTPError("HTTP response has invalid status code")
 		
 		self.status_code = int(fields[1])
-		self.status_name = fields[2]
+		
+		self.status_name = ""
+		if len(fields) == 3:
+			self.status_name = fields[2]
 		
 
 class HTTPParser:
@@ -579,9 +583,12 @@ class HTTPServerClient:
 				parser.update(data)
 			
 			response = await self.handle_request(parser.message)
-			if response is not None:
-				logger.info("Sending HTTP response (%i)", response.status_code)
-				await self.client.send(response.encode())
+			
+			logger.info("Sending HTTP response (%i)", response.status_code)
+			await self.client.send(response.encode())
+			
+			if response.upgrade:
+				await response.upgrade()
 		except Exception:
 			logger.exception("Failed to process HTTP request")
 	
@@ -598,7 +605,40 @@ class HTTPServerClient:
 			response = HTTPResponse(500)
 		
 		return response
+		
+		
+class HTTPRoute:
+	def __init__(self, router, path):
+		self.router = router
+		self.path = path
+	
+	def __enter__(self): return self
+	def __exit__(self, typ, val, tb):
+		self.router.remove(self.path)
+		
+		
+class HTTPRouter:
+	def __init__(self):
+		self.routes = {}
 
+	def route(self, path, handler):
+		if path in self.routes:
+			raise ValueError("Path is already routed")
+		
+		self.routes[path] = handler
+		return HTTPRoute(self, path)
+	
+	def remove(self, path):
+		del self.routes[path]
+	
+	async def handle(self, client, request):
+		if request.path not in self.routes:
+			logger.warning("HTTP router received unmapped request: %s" %request.path)
+			return HTTPResponse(404)
+		
+		handler = self.routes[request.path]
+		return await handler(client, request)
+		
 
 @contextlib.asynccontextmanager
 async def connect(url, context=None):
@@ -608,7 +648,7 @@ async def connect(url, context=None):
 		context = None
 	elif scheme == "https":
 		if context is None:
-			context = tls.TLSClientContext()
+			context = tls.TLSContext()
 	elif scheme is not None:
 		raise ValueError("Invalid HTTP url scheme: %s" %scheme)
 	
@@ -670,3 +710,9 @@ async def serve(handler, host="", port=0, context=None):
 	async with tls.serve(handle, host, port, context):
 		yield
 	logger.info("HTTP server is closed")
+
+@contextlib.asynccontextmanager
+async def serve_router(host="", port=0, context=None):
+	router = HTTPRouter()
+	async with serve(router.handle, host, port, context):
+		yield router
